@@ -32,11 +32,18 @@
 #include <kdirnotify.h>
 #include <kcharsets.h>
 
+#include <KRun>
+
 KNetAttach::KNetAttach( QWidget* parent )
     : Q3Wizard( parent ), Ui_KNetAttach()
 {
     setupUi( this );
 
+    QFont f = font();
+    f.setPointSizeF( f.pointSizeF() * 1.41 );
+    f.setBold( true );
+    setTitleFont( f );
+
     connect(_recent, SIGNAL(toggled(bool)), _recentConnectionName, SLOT(setEnabled(bool)));
     connect(_connectionName, SIGNAL(textChanged(const QString&)), this, SLOT(updateParametersPageStatus()));
     connect(_user, SIGNAL(textChanged(const QString&)), this, SLOT(updateParametersPageStatus()));
@@ -215,7 +222,7 @@ void KNetAttach::finished()
 	return;
     }
 
-    KToolInvocation::invokeBrowser(url.url());
+    KRun::runUrl(url, "inode/directory", this);
 
     QString name = _connectionName->text().trimmed();
 
