@@ -1,4 +1,5 @@
 /*
+ * Copyright 2008 Long Huynh Huu <long.upcase@googlemail.com>
  * Copyright 2007 Matthew Woehlke <mw_triad@users.sourceforge.net>
  * Copyright 2007 Casper Boemann <cbr@boemann.dk>
  *
@@ -18,8 +19,6 @@
  */
 
 #include "helper.h"
-#include "elements/scrollbar.h"
-#include "elements/progressbar.h"
 
 #include <KColorUtils>
 #include <KColorScheme>
@@ -36,6 +35,8 @@ OxygenStyleHelper::OxygenStyleHelper(const QByteArray &componentName)
 {
     // optimize for repainting of dock contents, which saves memory
     m_dockFrameCache.setMaxCost(1);
+    // we won't store many tilesets, because one size fits all
+    m_scrollHoleCache.setMaxCost(10);
 }
 
 QColor OxygenStyleHelper::calcMidColor(const QColor &color) const
@@ -52,10 +53,8 @@ void OxygenStyleHelper::invalidateCaches()
     m_holeFlatCache.clear();
     m_slopeCache.clear();
     m_slitCache.clear();
-    m_verticalScrollBarCache.clear();
-    m_horizontalScrollBarCache.clear();
-    m_progressBarCache.clear();
     m_dockFrameCache.clear();
+    m_scrollHoleCache.clear();
     OxygenHelper::invalidateCaches();
 }
 
@@ -485,7 +484,10 @@ TileSet *OxygenStyleHelper::slope(const QColor &color, double shade, int size)
         p.setCompositionMode(QPainter::CompositionMode_DestinationIn);
         p.drawRect(0, 9, 28, 19);
 
+        p.end();
+
         tileSet = new TileSet(pixmap, size, size, size*2, 2);
+
         m_slopeCache.insert(key, tileSet);
     }
     return tileSet;
@@ -633,12 +635,12 @@ TileSet *OxygenStyleHelper::slitFocused(const QColor &glowColor)
 
     if (!tileSet)
     {
-        QImage tmpImg(9, 9, QImage::Format_ARGB32);
+        QPixmap pixmap(9,9);
         QPainter p;
 
-        tmpImg.fill(Qt::transparent);
+        pixmap.fill(Qt::transparent);
 
-        p.begin(&tmpImg);
+        p.begin(&pixmap);
         p.setPen(Qt::NoPen);
         p.setRenderHint(QPainter::Antialiasing);
         QRadialGradient rg = QRadialGradient(4.5, 4.5, 4.5, 4.5, 4.5);
@@ -651,65 +653,15 @@ TileSet *OxygenStyleHelper::slitFocused(const QColor &glowColor)
         p.setBrush(rg);
         p.drawEllipse(QRectF(0, 0, 9, 9));
 
-        tileSet = new TileSet(QPixmap::fromImage(tmpImg), 4, 4, 1, 1);
+        p.end();
+
+        tileSet = new TileSet(pixmap, 4, 4, 1, 1);
 
         m_slitCache.insert(key, tileSet);
     }
     return tileSet;
 }
 
-TileSet *OxygenStyleHelper::verticalScrollBar(const QColor &color, int width, int offset, int size)
-{
-    size = (size*4)/3; // this code is writetn wrong :-), with base size == 8, not 6
-    offset %= (size * 4);
-
-    quint64 key = (quint64(color.rgba()) << 32) | (width<<22) | (offset<<10) | size;
-    TileSet *tileSet = m_verticalScrollBarCache.object(key);
-    if (!tileSet)
-    {
-        tileSet = OxygenScrollbar(color, _contrast).vertical(size, width, offset);
-        m_verticalScrollBarCache.insert(key, tileSet);
-    }
-    return tileSet;
-}
-
-TileSet *OxygenStyleHelper::horizontalScrollBar(const QColor &color, int width, int offset, int size)
-{
-    size = (size*4)/3; // this code is writetn wrong :-), with base size == 8, not 6
-    offset %= (size * 4);
-
-    quint64 key = (quint64(color.rgba()) << 32) | (width<<12) | offset;
-    TileSet *tileSet = m_horizontalScrollBarCache.object(key);
-    if (!tileSet)
-    {
-        tileSet = OxygenScrollbar(color, _contrast).horizontal(size, width, offset);
-        m_horizontalScrollBarCache.insert(key, tileSet);
-    }
-    return tileSet;
-}
-
-TileSet *OxygenStyleHelper::progressBar(const QColor &color, QRect rect,  Qt::Orientation orient, int size)
-{
-    size = (size*4)/3; // this code is writetn wrong :-), with base size == 8, not 6
-    int width;
-    if (orient == Qt::Horizontal)
-        width = rect.height();
-    else
-        width = rect.width();
-
-    quint64 key = (quint64(color.rgba()) << 32) | (width<<1) | (orient == Qt::Horizontal);
-    TileSet *tileSet = m_progressBarCache.object(key);
-    if (!tileSet)
-    {
-        if (orient == Qt::Horizontal)
-            tileSet = OxygenProgressBar(color, _contrast).horizontal(size, width);
-        else
-            tileSet = OxygenProgressBar(color, _contrast).vertical(size, width);
-        m_progressBarCache.insert(key, tileSet);
-    }
-    return tileSet;
-}
-
 TileSet *OxygenStyleHelper::dockFrame(const QColor &color, int width)
 {
     quint64 key = quint64(color.rgba()) << 32 | width;
@@ -744,22 +696,100 @@ TileSet *OxygenStyleHelper::dockFrame(const QColor &color, int width)
         lg.setColorAt(0.9, QColor(0,0,0,0));
         lg.setColorAt(1.0, light);
         p.setPen(QPen(lg,1));
-        p.drawRoundedRect(rect.adjusted(0,-1,0,-1),5,5);
-        p.drawRoundedRect(rect.adjusted(2,1,-2,-2),5,5);
+        p.drawRoundedRect(rect.adjusted(0,-1,0,-1),4,5);
+        p.drawRoundedRect(rect.adjusted(2,1,-2,-2),4,5);
 
         lg.setColorAt(0.0, dark);
         lg.setColorAt(0.1, QColor(0,0,0,0));
         lg.setColorAt(0.9, QColor(0,0,0,0));
         lg.setColorAt(1.0, dark);
         p.setPen(QPen(lg, 1));
-        p.drawRoundedRect(rect.adjusted(1,0,-1,-2),5,5);
+        p.drawRoundedRect(rect.adjusted(1,0,-1,-2),4,5);
 
         // top and bottom border
         drawSeparator(&p, QRect(0,0,w,2), color, Qt::Horizontal);
         drawSeparator(&p, QRect(0,h-2,w,2), color, Qt::Horizontal);
 
+        p.end();
+
         tileSet = new TileSet(pm, 4, 4, w-8, h-8);
+
         m_dockFrameCache.insert(key, tileSet);
     }
     return tileSet;
 }
+
+TileSet *OxygenStyleHelper::scrollHole(const QColor &color, Qt::Orientation orientation)
+{
+    quint64 key = quint64(color.rgba()) << 32 | (orientation == Qt::Horizontal);
+    TileSet *tileSet = m_scrollHoleCache.object(key);
+    if (!tileSet)
+    {
+        QPixmap pm(15, 15);
+        pm.fill(Qt::transparent);
+
+        QPainter p(&pm);
+
+        QColor dark = calcDarkColor(color);
+        QColor light = calcLightColor(color);
+        QColor shadow = calcShadowColor(color);
+        // use space for white border
+        QRect r = QRect(0,0,15,15);
+        QRect rect = r.adjusted(1, 0, -1, -1);
+        int shadowWidth = (orientation == Qt::Horizontal) ? 3 : 2;
+
+        p.setRenderHints(QPainter::Antialiasing);
+        p.setBrush(dark);
+        p.setPen(Qt::NoPen);
+
+        // base
+        p.drawRoundedRect(rect, 4.5, 4.5);
+
+        // slight shadow across the whole hole
+        QLinearGradient shadowGradient(rect.topLeft(),
+                orientation == Qt::Horizontal ?
+                    rect.bottomLeft()
+                :   rect.topRight());
+        shadowGradient.setColorAt(0.0, alphaColor(shadow, 0.1));
+        shadowGradient.setColorAt(0.6, Qt::transparent);
+        p.setBrush(shadowGradient);
+        p.drawRoundedRect(rect, 4.5, 4.5);
+
+        // strong shadow
+        // left
+        QLinearGradient l1 = QLinearGradient(rect.topLeft(), rect.topLeft()+QPoint(shadowWidth,0));
+        l1.setColorAt(0.0, alphaColor(shadow, orientation == Qt::Horizontal ? 0.3 : 0.2));
+        l1.setColorAt(0.5, alphaColor(shadow, orientation == Qt::Horizontal ? 0.1 : 0.1));
+        l1.setColorAt(1.0, Qt::transparent);
+        p.setBrush(l1);
+        p.drawRoundedRect(QRect(rect.topLeft(), rect.bottomLeft()+QPoint(shadowWidth,0)), 4.5, 4.5);
+        // right
+        l1 = QLinearGradient(rect.topRight(), rect.topRight()-QPoint(shadowWidth,0));
+        l1.setColorAt(0.0, alphaColor(shadow, orientation == Qt::Horizontal ? 0.3 : 0.2));
+        l1.setColorAt(0.5, alphaColor(shadow, orientation == Qt::Horizontal ? 0.1 : 0.1));
+        l1.setColorAt(1.0, Qt::transparent);
+        p.setBrush(l1);
+        p.drawRoundedRect(QRect(rect.topRight()-QPoint(shadowWidth,0), rect.bottomRight()), 4.5, 4.5);
+        //top
+        l1 = QLinearGradient(rect.topLeft(), rect.topLeft()+QPoint(0,3));
+        l1.setColorAt(0.0, alphaColor(shadow, 0.3));
+        l1.setColorAt(1.0, Qt::transparent);
+        p.setBrush(l1);
+        p.drawRoundedRect(QRect(rect.topLeft(),rect.topRight()+QPoint(0,3)), 4.5, 4.5);
+
+        // light border
+        QLinearGradient borderGradient(r.topLeft()+QPoint(0,r.height()/2-1), r.bottomLeft());
+        borderGradient.setColorAt(0.0, Qt::transparent);
+        borderGradient.setColorAt(1.0, alphaColor(light, 0.8));
+        p.setPen( QPen(borderGradient, 1.0) );
+        p.setBrush(Qt::NoBrush);
+        p.drawRoundedRect(r.adjusted(0.5,0,-0.5,0), 5.0, 5.0);
+
+        p.end();
+
+        tileSet = new TileSet(pm, 7, 7, 1, 1);
+
+        m_scrollHoleCache.insert(key, tileSet);
+    }
+    return tileSet;
+}
