@@ -20,6 +20,7 @@
 */
 
 #include "ktimezoned.moc"
+#include "ktimezonedbase.moc"
 
 #include <climits>
 #include <cstdlib>
@@ -57,8 +58,8 @@ const char ZONE_TAB_CACHE[] = "ZonetabCache";  // type of cached simulated zone.
 const char LOCAL_ZONE[]     = "LocalZone";     // name of local time zone
 
 
-KTimeZoned::KTimeZoned(QObject* parent, const QList<QVariant>&)
-  : KDEDModule(parent),
+KTimeZoned::KTimeZoned(QObject* parent, const QList<QVariant>& l)
+  : KTimeZonedBase(parent, l),
     mSource(0),
     mZonetabWatch(0),
     mDirWatch(0)
@@ -76,14 +77,6 @@ KTimeZoned::~KTimeZoned()
     mDirWatch = 0;
 }
 
-void KTimeZoned::initialize(bool reinit)
-{
-    // If we reach here, the module has already been constructed and therefore
-    // initialized. So only do anything if reinit is true.
-    if (reinit)
-        init(true);
-}
-
 void KTimeZoned::init(bool restart)
 {
     if (restart)
@@ -97,11 +90,6 @@ void KTimeZoned::init(bool restart)
         mDirWatch = 0;
     }
 
-#ifdef Q_OS_WIN
-    // On Windows, HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Time Zones
-    // holds the time zone database. The TZI binary value is the TIME_ZONE_INFORMATION structure.
-#else
-    // For Unix, read zone.tab.
     KConfig config(QLatin1String("ktimezonedrc"));
     if (restart)
         config.reparseConfiguration();
@@ -111,6 +99,9 @@ void KTimeZoned::init(bool restart)
     mConfigLocalZone = group.readEntry(LOCAL_ZONE);
     QString ztc      = group.readEntry(ZONE_TAB_CACHE, QString());
     mZoneTabCache    = (ztc == "Solaris") ? Solaris : NoCache;
+
+    // For Unix, read zone.tab.
+
     QString oldZoneinfoDir = mZoneinfoDir;
     QString oldZoneTab     = mZoneTab;
     CacheType oldCacheType = mZoneTabCache;
@@ -164,7 +155,6 @@ void KTimeZoned::init(bool restart)
     mZonetabWatch = new KDirWatch(this);
     mZonetabWatch->addFile(mZoneTab);
     connect(mZonetabWatch, SIGNAL(dirty(const QString&)), SLOT(zonetab_Changed(const QString&)));
-#endif
 
     // Find the local system time zone and set up file monitors to detect changes
     findLocalZone();
@@ -401,7 +391,7 @@ if (!mLocalZone.isEmpty()) kDebug(1221)<<"/etc/default/init: "<<mLocalZone;
         int bestOffset = INT_MAX;
         KSystemTimeZoneSource::startParseBlock();
         const KTimeZones::ZoneMap zmap = mZones.zones();
-        for (KTimeZones::ZoneMap::ConstIterator it = zmap.begin(), end = zmap.end();  it != end;  ++it)
+        for (KTimeZones::ZoneMap::ConstIterator it = zmap.constBegin(), end = zmap.constEnd();  it != end;  ++it)
         {
             KTimeZone zone = it.value();
             int candidateOffset = qAbs(zone.currentOffset(Qt::LocalTime));
@@ -678,7 +668,7 @@ bool KTimeZoned::matchZoneFile(const QString &path)
              */
             QString country = KGlobal::locale()->country().toUpper();
             const KTimeZones::ZoneMap zmap = mZones.zones();
-            for (KTimeZones::ZoneMap::ConstIterator zit = zmap.begin(), zend = zmap.end();  zit != zend;  ++zit)
+            for (KTimeZones::ZoneMap::ConstIterator zit = zmap.constBegin(), zend = zmap.constEnd();  zit != zend;  ++zit)
             {
                 KTimeZone tzone = zit.value();
                 if (tzone.countryCode() == country)
@@ -697,7 +687,7 @@ bool KTimeZoned::matchZoneFile(const QString &path)
         {
             // Look for a checksum match with the cached checksum values
             MD5Map oldChecksums = mMd5Sums;   // save a copy of the existing checksums
-            for (it5 = mMd5Sums.begin(), end5 = mMd5Sums.end();  it5 != end5;  ++it5)
+            for (it5 = mMd5Sums.constBegin(), end5 = mMd5Sums.constEnd();  it5 != end5;  ++it5)
             {
                 if (it5.value() == referenceMd5Sum)
                 {
@@ -720,7 +710,7 @@ bool KTimeZoned::matchZoneFile(const QString &path)
                 // Continue building missing entries in the cache on the assumption that
                 // we haven't previously looked at the zoneinfo file which matches.
                 const KTimeZones::ZoneMap zmap = mZones.zones();
-                for (KTimeZones::ZoneMap::ConstIterator zit = zmap.begin(), zend = zmap.end();  zit != zend;  ++zit)
+                for (KTimeZones::ZoneMap::ConstIterator zit = zmap.constBegin(), zend = zmap.constEnd();  zit != zend;  ++zit)
                 {
                     KTimeZone zone = zit.value();
                     zoneName = zone.name();
@@ -742,12 +732,12 @@ bool KTimeZoned::matchZoneFile(const QString &path)
                 // Didn't find the file, so presumably a previously cached checksum must
                 // have changed. Delete all the old checksums.
                 MD5Map::ConstIterator mit;
-                MD5Map::ConstIterator mend = oldChecksums.end();
-                for (mit = oldChecksums.begin();  mit != mend;  ++mit)
+                MD5Map::ConstIterator mend = oldChecksums.constEnd();
+                for (mit = oldChecksums.constBegin();  mit != mend;  ++mit)
                     mMd5Sums.remove(mit.key());
 
                 // And recalculate the old checksums
-                for (mit = oldChecksums.begin(); mit != mend; ++mit)
+                for (mit = oldChecksums.constBegin(); mit != mend; ++mit)
                 {
                     zoneName = mit.key();
                     QString candidateMd5Sum = calcChecksum(zoneName, referenceSize);
@@ -831,8 +821,8 @@ bool KTimeZoned::checkDefaultInit()
 // Check whether the checksum for a time zone matches a given saved checksum.
 KTimeZone KTimeZoned::compareChecksum(const KTimeZone &zone, const QString &referenceMd5Sum, qlonglong size)
 {
-    MD5Map::ConstIterator it5 = mMd5Sums.find(zone.name());
-    if (it5 == mMd5Sums.end())
+    MD5Map::ConstIterator it5 = mMd5Sums.constFind(zone.name());
+    if (it5 == mMd5Sums.constEnd())
     {
         // No checksum has been computed yet for this zone file.
         // Compute it now.
