@@ -21,6 +21,10 @@
 #include "devicepreference.h"
 
 #include <QtCore/QList>
+#include <QtDBus/QDBusConnection>
+#include <QtDBus/QDBusReply>
+#include <QtDBus/QDBusInterface>
+#include <QtDBus/QDBusMessage>
 #include <QtGui/QApplication>
 #include <QtGui/QPainter>
 #include <QtGui/QItemDelegate>
@@ -28,11 +32,11 @@
 #include <QtGui/QVBoxLayout>
 #include <QtGui/QHeaderView>
 
+#include <Phonon/AudioOutput>
+#include <Phonon/MediaObject>
 #include <phonon/backendcapabilities.h>
 #include <phonon/objectdescription.h>
 #include <phonon/phononnamespace.h>
-#include "../libkaudiodevicelist/audiodeviceenumerator.h"
-#include "../libkaudiodevicelist/audiodevice.h"
 #include "qsettingsgroup_p.h"
 #include "globalconfig_p.h"
 #include <kfadewidgeteffect.h>
@@ -83,109 +87,6 @@ class CategoryItem : public QStandardItem {
         Phonon::Category m_cat;
 };
 
-class DeviceTreeDelegate : public QItemDelegate
-{
-    public:
-        DeviceTreeDelegate(QObject *parent = 0);
-        QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const;
-        void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
-};
-
-DeviceTreeDelegate::DeviceTreeDelegate(QObject *parent)
-    : QItemDelegate(parent)
-{
-}
-
-QSize DeviceTreeDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
-{
-    QVariant value = index.data(Qt::SizeHintRole);
-    if (value.isValid()) {
-        return qvariant_cast<QSize>(value);
-    }
-
-    QRect iconRect;
-    value = index.data(Qt::DecorationRole);
-    if (value.type() == QVariant::Icon) {
-        QIcon icon = qvariant_cast<QIcon>(value);
-        iconRect = QRect(QPoint(0, 0), icon.actualSize(option.decorationSize));
-    }
-
-    QRect textRect;
-    value = index.data(Qt::DisplayRole);
-    if (value.isValid()) {
-        QString text = value.toString();
-        const QSize size = option.fontMetrics.size(Qt::TextSingleLine, text);
-        const int textMargin = QApplication::style()->pixelMetric(QStyle::PM_FocusFrameHMargin) + 1;
-        textRect = QRect(0, 0, size.width() + 2 * textMargin, size.height());
-    }
-    QRect checkRect = rect(option, index, Qt::CheckStateRole);
-
-    doLayout(option, &checkRect, &iconRect, &textRect, true);
-
-    return (iconRect | textRect | checkRect).size();
-}
-
-void DeviceTreeDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
-{
-    QStyleOptionViewItemV3 opt = setOptions(index, option);
-
-    const QStyleOptionViewItemV2 *v2 = qstyleoption_cast<const QStyleOptionViewItemV2 *>(&option);
-    const QStyleOptionViewItemV3 *v3 = qstyleoption_cast<const QStyleOptionViewItemV3 *>(&option);
-    opt.features = v2 ? v2->features : QStyleOptionViewItemV2::ViewItemFeatures(QStyleOptionViewItemV2::None);
-    opt.locale   = v3 ? v3->locale   : QLocale();
-    opt.widget   = v3 ? v3->widget   : 0;
-
-    painter->save();
-    // clip painter to the actual item - i.e. don't paint outside
-    painter->setClipRect(opt.rect);
-
-    QIcon icon;
-    QIcon::Mode mode = QIcon::Normal;
-    QIcon::State state = QIcon::Off;
-    QRect iconRect;
-    QVariant value = index.data(Qt::DecorationRole);
-    if (value.type() == QVariant::Icon) {
-        icon = qvariant_cast<QIcon>(value);
-        if (!(option.state & QStyle::State_Enabled)) {
-            mode = QIcon::Disabled;
-        } else if (option.state & QStyle::State_Selected) {
-            mode = QIcon::Selected;
-        }
-        if (option.state & QStyle::State_Open) {
-            state = QIcon::On;
-        }
-        iconRect = QRect(QPoint(0, 0), icon.actualSize(option.decorationSize, mode, state));
-    }
-
-    QString text;
-    QRect textRect;
-    value = index.data(Qt::DisplayRole);
-    if (value.isValid()) {
-        text = value.toString();
-        const QSize size = opt.fontMetrics.size(Qt::TextSingleLine, text);
-        const int textMargin = QApplication::style()->pixelMetric(QStyle::PM_FocusFrameHMargin) + 1;
-        textRect = QRect(0, 0, size.width() + 2 * textMargin, size.height());
-    }
-
-    QRect checkRect;
-    Qt::CheckState checkState = Qt::Unchecked;
-    value = index.data(Qt::CheckStateRole);
-    if (value.isValid()) {
-        checkState = static_cast<Qt::CheckState>(value.toInt());
-        checkRect = check(opt, opt.rect, value);
-    }
-
-    doLayout(opt, &checkRect, &iconRect, &textRect, false);
-
-    drawBackground(painter, opt, index);
-    drawCheck(painter, opt, checkRect, checkState);
-    icon.paint(painter, iconRect, option.decorationAlignment, mode, state);
-    drawDisplay(painter, opt, textRect, text);
-    drawFocus(painter, opt, textRect);
-
-    painter->restore();
-}
-
 /**
  * Need this to change the colors of the ListView if the Palette changed. With CSS set this won't
  * change automatically
@@ -201,10 +102,14 @@ void DevicePreference::changeEvent(QEvent *e)
 
 DevicePreference::DevicePreference(QWidget *parent)
     : QWidget(parent),
-    m_headerModel(0, 1, 0)
+    m_headerModel(0, 1, 0),
+    m_showingOutputModel(true),
+    m_media(0), m_output(0)
 {
     setupUi(this);
-    deviceList->setItemDelegate(new DeviceTreeDelegate(deviceList));
+    testPlaybackButton->setIcon(KIcon("media-playback-start"));
+    testPlaybackButton->setEnabled(false);
+    testPlaybackButton->setToolTip(i18n("Play a test sound on the selected device"));
     removeButton->setIcon(KIcon("list-remove"));
     deferButton->setIcon(KIcon("go-down"));
     preferButton->setIcon(KIcon("go-up"));
@@ -278,6 +183,8 @@ DevicePreference::DevicePreference(QWidget *parent)
     connect(showCheckBox, SIGNAL(stateChanged (int)), this, SIGNAL(changed()));
     connect(Phonon::BackendCapabilities::notifier(), SIGNAL(availableAudioOutputDevicesChanged()), SLOT(updateAudioOutputDevices()));
     connect(Phonon::BackendCapabilities::notifier(), SIGNAL(availableAudioCaptureDevicesChanged()), SLOT(updateAudioCaptureDevices()));
+    connect(Phonon::BackendCapabilities::notifier(), SIGNAL(capabilitiesChanged()), SLOT(updateAudioOutputDevices()));
+    connect(Phonon::BackendCapabilities::notifier(), SIGNAL(capabilitiesChanged()), SLOT(updateAudioCaptureDevices()));
 
     if (!categoryTree->currentIndex().isValid()) {
         categoryTree->setCurrentIndex(m_categoryModel.index(0, 0).child(1, 0));
@@ -301,20 +208,22 @@ void DevicePreference::updateDeviceList()
         } else {
             deviceList->setModel(m_captureModel[cat]);
         }
+        m_showingOutputModel = catItem->isOutputItem;
         if (cat == Phonon::NoCategory) {
             if (catItem->isOutputItem) {
-                m_headerModel.setHeaderData(0, Qt::Horizontal, i18n("Default Output Device Preference:"), Qt::DisplayRole);
+                m_headerModel.setHeaderData(0, Qt::Horizontal, i18n("Default Output Device Preference"), Qt::DisplayRole);
             } else {
-                m_headerModel.setHeaderData(0, Qt::Horizontal, i18n("Default Capture Device Preference:"), Qt::DisplayRole);
+                m_headerModel.setHeaderData(0, Qt::Horizontal, i18n("Default Capture Device Preference"), Qt::DisplayRole);
             }
         } else {
             if (catItem->isOutputItem) {
-                m_headerModel.setHeaderData(0, Qt::Horizontal, i18n("Output Device Preference for the '%1' Category:", Phonon::categoryToString(cat)), Qt::DisplayRole);
+                m_headerModel.setHeaderData(0, Qt::Horizontal, i18n("Output Device Preference for the '%1' Category", Phonon::categoryToString(cat)), Qt::DisplayRole);
             } else {
-                m_headerModel.setHeaderData(0, Qt::Horizontal, i18n("Capture Device Preference for the '%1' Category:", Phonon::categoryToString(cat)), Qt::DisplayRole);
+                m_headerModel.setHeaderData(0, Qt::Horizontal, i18n("Capture Device Preference for the '%1' Category", Phonon::categoryToString(cat)), Qt::DisplayRole);
             }
         }
     } else {
+        m_showingOutputModel = false;
         m_headerModel.setHeaderData(0, Qt::Horizontal, QString(), Qt::DisplayRole);
         deviceList->setModel(0);
     }
@@ -331,8 +240,42 @@ void DevicePreference::updateDeviceList()
 
 void DevicePreference::updateAudioCaptureDevices()
 {
-    //TODO
-    kFatal() << "not implemented";
+    kDebug();
+    const QList<Phonon::AudioCaptureDevice> list = availableAudioCaptureDevices();
+    QHash<int, Phonon::AudioCaptureDevice> hash;
+    foreach (const Phonon::AudioCaptureDevice &dev, list) {
+        hash.insert(dev.index(), dev);
+    }
+    for (int ii = 0; ii < captureCategoriesCount; ++ii) {
+        const int i = captureCategories[ii];
+        Phonon::AudioCaptureDeviceModel *model = m_captureModel.value(i);
+        Q_ASSERT(model);
+
+        QHash<int, Phonon::AudioCaptureDevice> hashCopy(hash);
+        QList<Phonon::AudioCaptureDevice> orderedList;
+        if (model->rowCount() > 0) {
+            QList<int> order = model->tupleIndexOrder();
+            foreach (int idx, order) {
+                if (hashCopy.contains(idx)) {
+                    orderedList << hashCopy.take(idx);
+                }
+            }
+            if (hashCopy.size() > 1) {
+                // keep the order of the original list
+                foreach (const Phonon::AudioCaptureDevice &dev, list) {
+                    if (hashCopy.contains(dev.index())) {
+                        orderedList << hashCopy.take(dev.index());
+                    }
+                }
+            } else if (hashCopy.size() == 1) {
+                orderedList += hashCopy.values();
+            }
+            model->setModelData(orderedList);
+        } else {
+            model->setModelData(list);
+        }
+    }
+    deviceList->resizeColumnToContents(0);
 }
 
 void DevicePreference::updateAudioOutputDevices()
@@ -492,6 +435,13 @@ void DevicePreference::save()
         QSettingsGroup generalGroup(&config, QLatin1String("General"));
         generalGroup.setValue(QLatin1String("HideAdvancedDevices"), !showCheckBox->isChecked());
     }
+    if (!m_removeOnApply.isEmpty()) {
+        QDBusMessage msg = QDBusMessage::createMethodCall("org.kde.kded", "/modules/phononserver",
+                "org.kde.PhononServer", "removeAudioDevices");
+        msg << QVariant::fromValue(m_removeOnApply);
+        QDBusConnection::sessionBus().send(msg);
+        m_removeOnApply.clear();
+    }
     {
         QSettingsGroup globalGroup(&config, QLatin1String("AudioOutputDevice"));
         const QList<int> noCategoryOrder = m_outputModel.value(Phonon::NoCategory)->tupleIndexOrder();
@@ -526,13 +476,13 @@ void DevicePreference::save()
 void DevicePreference::defaults()
 {
     {
-        QList<Phonon::AudioOutputDevice> list = availableAudioOutputDevices();
+        const QList<Phonon::AudioOutputDevice> list = availableAudioOutputDevices();
         for (int i = -1; i <= Phonon::LastCategory; ++i) {
             m_outputModel[i]->setModelData(list);
         }
     }
     {
-        QList<Phonon::AudioCaptureDevice> list = availableAudioCaptureDevices();
+        const QList<Phonon::AudioCaptureDevice> list = availableAudioCaptureDevices();
         for (int i = 0; i < captureCategoriesCount; ++i) {
             m_captureModel[captureCategories[i]]->setModelData(list);
         }
@@ -583,6 +533,36 @@ void DevicePreference::on_deferButton_clicked()
     }
 }
 
+template<Phonon::ObjectDescriptionType T>
+void DevicePreference::removeDevice(const Phonon::ObjectDescription<T> &deviceToRemove,
+        QMap<int, Phonon::ObjectDescriptionModel<T> *> *modelMap)
+{
+    QDBusInterface phononServer(QLatin1String("org.kde.kded"), QLatin1String("/modules/phononserver"),
+            QLatin1String("org.kde.PhononServer"));
+    QDBusReply<bool> reply = phononServer.call(QLatin1String("isAudioDeviceRemovable"), deviceToRemove.index());
+    if (!reply.isValid()) {
+        kError(600) << reply.error();
+        return;
+    }
+    if (!reply.value()) {
+        return;
+    }
+    m_removeOnApply << deviceToRemove.index();
+
+    // remove from all models, idx.row() is only correct for the current model
+    foreach (Phonon::ObjectDescriptionModel<T> *model, *modelMap) {
+        QList<Phonon::ObjectDescription<T> > data = model->modelData();
+        for (int row = 0; row < data.size(); ++row) {
+            if (data[row] == deviceToRemove) {
+                model->removeRows(row, 1);
+                break;
+            }
+        }
+    }
+    updateButtonsEnabled();
+    emit changed();
+}
+
 void DevicePreference::on_removeButton_clicked()
 {
     const QModelIndex idx = deviceList->currentIndex();
@@ -590,45 +570,12 @@ void DevicePreference::on_removeButton_clicked()
     QAbstractItemModel *model = deviceList->model();
     Phonon::AudioOutputDeviceModel *playbackModel = qobject_cast<Phonon::AudioOutputDeviceModel *>(model);
     if (playbackModel && idx.isValid()) {
-        const Phonon::AudioOutputDevice deviceToRemove = playbackModel->modelData(idx);
-        const QList<Phonon::AudioDevice> deviceList = Phonon::AudioDeviceEnumerator::availablePlaybackDevices();
-        foreach (Phonon::AudioDevice dev, deviceList) {
-            if (-dev.index() == deviceToRemove.index()) {
-                // remove from persistent store
-                if (dev.ceaseToExist()) {
-                    // remove from all models, idx.row() is only correct for the current model
-                    foreach (Phonon::AudioOutputDeviceModel *model, m_outputModel) {
-                        QList<Phonon::AudioOutputDevice> data = model->modelData();
-                        for (int row = 0; row < data.size(); ++row) {
-                            if (data[row] == deviceToRemove) {
-                                model->removeRows(row, 1);
-                                break;
-                            }
-                        }
-                    }
-                    updateButtonsEnabled();
-                    emit changed();
-                }
-            }
-        }
-        /*
+        removeDevice(playbackModel->modelData(idx), &m_outputModel);
     } else {
         Phonon::AudioCaptureDeviceModel *captureModel = qobject_cast<Phonon::AudioCaptureDeviceModel *>(model);
         if (captureModel && idx.isValid()) {
-            Phonon::AudioCaptureDevice deviceToRemove = captureModel->modelData(idx);
-            QList<Phonon::AudioDevice> deviceList = Phonon::AudioDeviceEnumerator::availableCaptureDevices();
-            foreach (Phonon::AudioDevice dev, deviceList) {
-                if (-dev.index() == deviceToRemove.index()) {
-                    // remove from persistent store
-                    if (dev.ceaseToExist()) {
-                        m_captureModel.removeRows(idx.row(), 1);
-                        updateButtonsEnabled();
-                        emit changed();
-                    }
-                }
-            }
+            removeDevice(captureModel->modelData(idx), &m_captureModel);
         }
-        */
     }
 
     deviceList->resizeColumnToContents(0);
@@ -757,6 +704,36 @@ void DevicePreference::on_showCheckBox_toggled()
     deviceList->resizeColumnToContents(0);
 }
 
+void DevicePreference::on_testPlaybackButton_toggled(bool down)
+{
+    if (down) {
+        QModelIndex idx = deviceList->currentIndex();
+        if (!idx.isValid() || !m_showingOutputModel) {
+            return;
+        }
+        const Phonon::AudioOutputDeviceModel *model = static_cast<const Phonon::AudioOutputDeviceModel *>(idx.model());
+        const Phonon::AudioOutputDevice &device = model->modelData(idx);
+        m_media = new Phonon::MediaObject(this);
+        m_output = new Phonon::AudioOutput(this);
+        m_output->setOutputDevice(device);
+
+        // just to be very sure that nothing messes our test sound up
+        m_output->setVolume(1.0);
+        m_output->setMuted(false);
+
+        Phonon::createPath(m_media, m_output);
+        connect(m_media, SIGNAL(finished()), testPlaybackButton, SLOT(toggle()));
+        m_media->setCurrentSource(KStandardDirs::locate("sound", "KDE-Sys-Log-In.ogg"));
+        m_media->play();
+    } else {
+        disconnect(m_media, SIGNAL(finished()), testPlaybackButton, SLOT(toggle()));
+        delete m_media;
+        m_media = 0;
+        delete m_output;
+        m_output = 0;
+    }
+}
+
 void DevicePreference::updateButtonsEnabled()
 {
     //kDebug() ;
@@ -765,11 +742,14 @@ void DevicePreference::updateButtonsEnabled()
         QModelIndex idx = deviceList->currentIndex();
         preferButton->setEnabled(idx.isValid() && idx.row() > 0);
         deferButton->setEnabled(idx.isValid() && idx.row() < deviceList->model()->rowCount() - 1);
-        removeButton->setEnabled(idx.isValid() && !(idx.flags()  & Qt::ItemIsEnabled));
+        removeButton->setEnabled(idx.isValid() && !(idx.flags() & Qt::ItemIsEnabled));
+        testPlaybackButton->setEnabled(m_showingOutputModel && idx.isValid() &&
+                (idx.flags() & Qt::ItemIsEnabled));
     } else {
         preferButton->setEnabled(false);
         deferButton->setEnabled(false);
         removeButton->setEnabled(false);
+        testPlaybackButton->setEnabled(false);
     }
 }
 
